*** Settings ***
Documentation  查询操作mysql数据库的关键字。引用请使用Resource  atBasicLibray/keywords/mysql.robot [ sc/mysql.html | mysql.robot ]
Library  ../mysqlLib.py

*** Keywords ***
操作数据库mysql数据
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句操作mysql数据。返回结果是影响的行数，Int类型。
    ...     英文关键字[ ../keywords/mysql.doc.html#Execute%20database%20mysql%20data | Execute database mysql data]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     返回结果是影响的行数，Int类型。
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Execute%20Sql%20With%20Config | at_execute_sql_with_config ] 方法。
    [Arguments]                             ${sql}   ${config}
    ${result}=  at execute sql with config  ${sql}   ${config}
    [Return]  ${result}

Execute database mysql data
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句操作mysql数据。返回结果是影响的行数，Int类型。
    ...     中文关键字[ ../keywords/mysql.doc.html#操作数据库mysql数据 | 操作数据库mysql数据]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     返回结果是影响的行数，Int类型。
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Execute%20Mysql%20Sql | at_execute_sql_with_config ] 方法。
    [Arguments]                             ${sql}   ${config}
    ${result}=  at execute sql with config  ${sql}   ${config}
    [Return]  ${result}

查询数据库mysql数据
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句查询mysql数据。返回结果是字典列表。
    ...     英文关键字[ ../keywords/mysql.doc.html#Query%20database%20mysql%20data | Query database mysql data]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Query%20Mysql%20With%20Config | at_query_mysql_with_config ] 方法。
    ...     返回结果是字典列表。比如: [{'aa':"123","bb":"456"},{"aa":"789","bb":"123"}]。如果没有任何数据，会返回None。
    [Arguments]                             ${sql}   ${config}
    ${result}=  at query mysql with config  ${sql}   ${config}
    [Return]  ${result}

Query database mysql data
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句查询mysql数据。返回结果是字典列表。
    ...     中文关键字[ ../keywords/mysql.doc.html#查询数据库mysql数据 | 查询数据库mysql数据]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Query%20Mysql%20With%20Config | at_query_mysql_with_config ] 方法。
    ...     返回结果是字典列表。比如: [{'aa':"123","bb":"456"},{"aa":"789","bb":"123"}]。如果没有任何数据，会返回None。
    [Arguments]                             ${sql}   ${config}
    ${result}=  at query mysql with config  ${sql}   ${config}
    [Return]  ${result}

查询数据库mysql一条数据
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句查询mysql一条数据。返回结果是字典。
    ...     英文关键字[ ../keywords/mysql.doc.html#Query%20database%20Only%20One%20mysql%20data | Query database only one mysql data]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     返回结果是字典。比如: {'aa':"123","bb":"456"}。如果没有任何数据，会返回None。
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Query%20One%20With%20Config | at_query_one_with_config] 方法。
    [Arguments]   ${sql}   ${config}
    ${result}=  at query one with config  ${sql}   ${config}
    [Return]  ${result}

Query database only one mysql data
    [Documentation]  功能:通过含有host,port,user,password,db等配置的字典,用sql语句查询mysql一条数据。返回结果是字典。
    ...     中文关键字[ ../keywords/mysql.doc.html#查询数据库mysql一条数据 | 查询数据库mysql一条数据]。
    ...     第一个参数是SQL语句，String类型,
    ...     第二个参数是含有host,port,user,password,db等配置的字典，字典类型,比如
    ...     {"host":"127.0.0.1","port":3306,"user":"root","password":"password","db":"database","charset":"utf8mb4"}
    ...     返回结果是字典。比如: {'aa':"123","bb":"456"}。如果没有任何数据，会返回None。
    ...     具体代码请参考atBasicLibrary/mysqlLib.py的 [../library/mysqlLib.py.doc.html#At%20Query%20One%20With%20Config | at_query_one_with_config] 方法。
    [Arguments]   ${sql}   ${config}
    ${result}=  at query one with config  ${sql}   ${config}
    [Return]  ${result}
